@@ -15,7 +15,7 @@ the :mod:`amoco.system` package.
 """
 
 from amoco.arch.core import Bits
-from amoco.ui.views import execView
+from amoco.ui.views import execView, dataView
 from amoco.logger import Log
 
 logger = Log(__name__)
@@ -36,7 +36,9 @@ class CoreExec(object):
     Attributes:
         bin: the program executable format object. Currently supported formats
              are provided in :mod:`system.elf` (Elf32/64), :mod:`system.pe` (PE)
-             and :mod:`system.utils` (HEX/SREC).
+             :mod:`system.macho` (Mach-O) :mod:`system.structs.HEX` (HEX),
+             :mod:`system.structs.SREC` (SREC), and raw "shellcode-like" format
+             in :class:`system.core.shellcode`.
 
         cpu: reference to the architecture cpu module, which provides a generic
              access to the PC() program counter and
@@ -99,6 +101,10 @@ class CoreExec(object):
         if self.cpu is None:
             logger.error("no cpu imported")
             raise ValueError
+        if "mmap" in kargs:
+            mmap = kargs[mmap]
+        else:
+            mmap = self.state.mmap
         maxlen = self.cpu.disassemble.maxlen
         if isinstance(vaddr, int):
             addr = self.cpu.cst(vaddr, self.cpu.PC().size)
@@ -108,7 +114,7 @@ class CoreExec(object):
         else:
             addr = vaddr
         try:
-            istr = self.state.mmap.read(vaddr, maxlen)
+            istr = mmap.read(vaddr, maxlen)
         except MemoryError as e:
             logger.verbose("vaddr %s is not mapped" % addr)
             raise MemoryError(e)
@@ -129,10 +135,6 @@ class CoreExec(object):
         else:
             if i.address is None:
                 i.address = addr
-            xsz = i.misc["xsz"] or 0
-            if xsz > 0:
-                xdata = self.state.mmap.read(vaddr + i.length, xsz)
-                i.xdata(i, xdata)
             return i
 
     def symbol_for(self,address):
@@ -247,6 +249,14 @@ class CoreExec(object):
         "get 8-bit unsigned int expression of current state(loc)"
         return self.getx(loc)
 
+    def get_cstr(self, loc):
+        "get null-terminated unsigned char array of current state(loc)"
+        A = [self.get_uint8(loc)]
+        while A[-1]!=0:
+            loc += 1
+            A.append(self.get_uint8(loc))
+        return bytes(A)
+
 
 # ------------------------------------------------------------------------------
 
@@ -282,6 +292,8 @@ class BinFormat(object):
     is_ELF = False
     is_PE = False
     is_MachO = False
+    is_HEX = False
+    is_SREC = False
     basemap = None
     symtab = None
     strtab = None
@@ -304,11 +316,33 @@ class BinFormat(object):
         return (None, 0, 0)
 
 
-class DataIO(BinFormat):
+class shellcode(BinFormat):
+    """
+    This is the most basic file format for executable binary code. It
+    provides zero information about the targeted architecture, entrypoints, or
+    any other data or code dependencies.
+    """
+    def __init__(self,dataio):
+        self.data = dataio
+
+    @property
+    def entrypoints(self):
+        return [0]
+
+    @property
+    def filename(self):
+        return self.data.name
+
+    @property
+    def dataio(self):
+        return self.data
+
+
+class DataIO(object):
     """
     This class simply wraps a binary file or a bytes string and implements
     both the file and bytes interface. It allows an input to be provided as
-    files of bytes and manipulated as either a file or a bytes object.
+    files of bytes and manipulated indifferently as a file or a bytes object.
     """
 
     def __init__(self, f):
@@ -318,10 +352,13 @@ class DataIO(BinFormat):
             self.f = BytesIO(f)
         else:
             self.f = f
+        self.view = dataView(dataio=self)
 
     def __getitem__(self, i):
         stay = self.f.tell()
-        sta = i.start or stay
+        sta = i.start
+        if sta is None:
+            sta = stay
         self.f.seek(sta, 0)
         if i.stop is None:
             data = self.f.read()
@@ -330,6 +367,13 @@ class DataIO(BinFormat):
         self.f.seek(stay, 0)
         return data
 
+    def size(self):
+        stay = self.f.tell()
+        self.f.seek(0,2)
+        sz = self.f.tell()
+        self.f.seek(stay,0)
+        return sz
+
     def read(self, size=-1):
         return self.f.read(size)
 
@@ -406,7 +450,6 @@ class DataIO(BinFormat):
     def softspace(self):
         return self.f.softspace
 
-
 # ------------------------------------------------------------------------------
 def read_program(filename):
     """
@@ -434,7 +477,7 @@ def read_program(filename):
         p = elf.Elf(f)
         logger.info("ELF format detected")
         return p
-    except elf.ElfError:
+    except (elf.StructureError,elf.ElfError):
         f.seek(0)
         logger.debug("ElfError raised for %s" % f.name)
 
@@ -445,7 +488,7 @@ def read_program(filename):
         p = pe.PE(f)
         logger.info("PE format detected")
         return p
-    except pe.PEError:
+    except (pe.StructureError,pe.PEError):
         f.seek(0)
         logger.debug("PEError raised for %s" % f.name)
 
@@ -456,32 +499,33 @@ def read_program(filename):
         p = macho.MachO(f)
         logger.info("Mach-O format detected")
         return p
-    except macho.MachOError:
+    except (macho.StructureError,macho.MachOError):
         f.seek(0)
         logger.debug("MachOError raised for %s" % f.name)
 
     try:
-        from amoco.system import utils
+        from amoco.system.structs.HEX import HEX,HEXError
 
         # open file as a HEX object:
-        p = utils.HEX(f)
+        p = HEX(f)
         logger.info("HEX format detected")
         return p
-    except utils.FormatError:
+    except HEXError:
         f.seek(0)
         logger.debug(" HEX FormatError raised for %s" % f.name)
 
     try:
+        from amoco.system.structs.SREC import SREC,SRECError
         # open file as a SREC object:
-        p = utils.SREC(f)
+        p = SREC(f)
         logger.info("SREC format detected")
         return p
-    except utils.FormatError:
+    except SRECError:
         f.seek(0)
         logger.debug(" SREC FormatError raised for %s" % f.name)
 
     logger.warning("unknown format")
-    return f
+    return shellcode(f)
 
 
 # ------------------------------------------------------------------------------
@@ -527,15 +571,19 @@ class DefineLoader(object):
         return loader
 
 
-def load_program(f, cpu=None):
+def load_program(f, cpu=None, loader=None):
     """
-    Detects program format header (ELF/PE/Mach-O/HEX/SREC),
+    Detects program format header (ELF/PE/Mach-O/HEX/SREC), or consider
+    the input as a raw "shellcode" if no supported format is recognized,
     and *maps* the program in abstract memory,
     loading the associated "system" (linux/win) and "arch" (x86/arm),
     based header informations.
 
     Arguments:
         f (str): the program filename or string of bytes.
+        cpu (module): optionally, the cpu python module in the program
+                      header does not provide it (HEX/SREC/shellcode)
+        loader (str): optionally, the loader name (a key of Loaders dict.)
 
     Returns:
         a Task, ELF/PE (old CoreExec interfaces) or RawExec instance.
@@ -558,6 +606,8 @@ def load_program(f, cpu=None):
     logger.verbose("--- create task ---")
 
     Loaders = DefineLoader.LOADERS
+    if loader is not None and (loader in Loaders):
+        x = Loaders[loader](p)
     if p.is_ELF:
         try:
             x = Loaders["elf"][p.Ehdr.e_machine](p)
