@@ -11,7 +11,7 @@ system/elf.py
 The system elf module implements Elf classes for both 32/64bits executable format.
 """
 from amoco.system.core import BinFormat
-from amoco.system.structs import Consts, StructDefine
+from amoco.system.structs import Consts, StructDefine, StructureError
 from amoco.system.structs import StructFormatter, token_constant_fmt, token_address_fmt
 
 from amoco.logger import Log
@@ -63,6 +63,10 @@ class Elf(BinFormat):
     def header(self):
         return self.Ehdr
 
+    @property
+    def dataio(self):
+        return self.__file
+
     def __init__(self, f):
         self.__file = f
         self.Ehdr = Ehdr(f)
@@ -106,13 +110,13 @@ class Elf(BinFormat):
                 logger.verbose("exception raised while parsing section(s)")
 
         # read section's name string table:
+        for i, s in enumerate(self.Shdr):
+            s.name = ".s%d" % i
         n = self.Ehdr.e_shstrndx
         if n != SHN_UNDEF and n in range(len(self.Shdr)):
             S = self.Shdr[self.Ehdr.e_shstrndx]
             if S.sh_type != SHT_STRTAB:
                 logger.verbose("section names not a string table")
-                for i, s in enumerate(self.Shdr):
-                    s.name = ".s%d" % i
             else:
                 from codecs import decode
 
@@ -472,7 +476,7 @@ class IDENT(StructFormatter):
         if data:
             self.unpack(data)
 
-    def unpack(self, data, offset=0):
+    def unpack(self, data, offset=0, psize=0):
         StructFormatter.unpack(self, data, offset)
         if self.ELFMAG0 != 0x7F or self.ELFMAG != b"ELF":
             raise ElfError("Wrong magic number, not an ELF file ?")
@@ -539,7 +543,7 @@ class Ehdr(StructFormatter):
         if data:
             self.unpack(data)
 
-    def unpack(self, data, offset=0):
+    def unpack(self, data, offset=0, psize=0):
         f0 = self.fields[0]
         self._v.e_ident = f0.unpack(data, offset)
         offset += f0.size()
