@@ -8,7 +8,25 @@
 render.py
 =========
 
-This module ...
+This module implements amoco's pygments interface to allow pretty printed
+outputs of tables of tokens built from amoco's expressions and instructions.
+The rendered texts are used as main inputs for graphic engines to build
+their own views' objects.
+
+A token is a tuple (t,s) where t is a Token type and s is a python string.
+The highlight method uses the Token type to decorate the string s such that
+the targeted renderer is able to show the string with foreground/background
+colors and bold/underline/etc stroke attributes.
+
+The API of this module is essentially the vltable class which implements its
+str interface by calls to the highlight function, wrapping the pygments formatters
+to allow colored output.
+Note that more specialized formatting like HTML tables or even LaTeX blobs is
+also possible.
+
+If the pygments package is not found, all output default to a kind of
+"NullFormatter" that will just ignore input tokens' types and just assemble lines
+into undercorated unicode strings.
 """
 
 from io import BytesIO as StringIO
@@ -55,7 +73,7 @@ except ImportError:
 else:
     logger.verbose("pygments package imported")
     has_pygments = True
-
+    # define default dark style:
     dark = {
         Token.Literal  : "#fff",
         Token.Address  : "#fb0",
@@ -76,6 +94,8 @@ else:
         Token.Column   : "#000",
     }
     S = {}
+    # define sub-tokens with Mark/Taint/Hide atrribute,
+    # allowing to set tokens types like Token.Register.Taint
     for k in dark.keys():
         S[getattr(k,'Mark')]  = "bg:#224"
         S[getattr(k,'Taint')] = "bg:#422"
@@ -85,7 +105,7 @@ else:
     class DarkStyle(Style):
         default_style = ""
         styles = dark
-
+    # define default light style:
     light = {
         Token.Literal  : "",
         Token.Address  : "#c30",
@@ -115,9 +135,9 @@ else:
     class LightStyle(Style):
         default_style = ""
         styles = light
-
+    # the default style is dark:
     DefaultStyle = DarkStyle
-
+    # define supported formatters:
     Formats = {
         "Null"         : NullFormatter(encoding="utf-8"),
         "Terminal"     : TerminalFormatter(style=DefaultStyle, encoding="utf-8"),
@@ -125,10 +145,21 @@ else:
         "TerminalDark" : Terminal256Formatter(style=DarkStyle, encoding="utf-8"),
         "TerminalLight": Terminal256Formatter(style=LightStyle, encoding="utf-8"),
         "Html"         : HtmlFormatter(style=LightStyle, encoding="utf-8"),
+        "HtmlDark"     : HtmlFormatter(style=DarkStyle, encoding="utf-8"),
     }
 
 
 def highlight(toks, formatter=None, outfile=None):
+    """
+    Pretty prints a list of tokens using optionally
+    a given formatter and an output io buffer.
+
+    If no explicit formatter is given, use the formatter from configuration
+    or the Null formatter if not specified in the amoco configuration.
+    If no output io buffer is given, a local StringIO is used.
+
+    The returned value is a decorated python string.
+    """
     formatter = formatter or Formats.get(conf.UI.formatter,"Null")
     if isinstance(formatter, str):
         formatter = Formats[formatter]
@@ -156,7 +187,7 @@ def TokenListJoin(j, lst):
     if isinstance(j, str):
         j = (Token.Literal, j)
     # init output list:
-    res = lst[0]
+    res = lst[0] if len(lst)>0 else []
     if not isinstance(res,list):
         res = [res]
     for x in lst[1:]:
@@ -177,7 +208,7 @@ def LambdaTokenListJoin(j,f):
 
 class vltable(object):
     """
-    variable length table relies on pygments to pretty print tabulated data.
+    A variable length table relies on pygments to pretty print tabulated data.
 
     Arguments:
         rows (list): optional argument with initial list of tokenrows.
