@@ -8,7 +8,18 @@
 views.py
 ========
 
-This module ...
+This module implements the high-level (engines agnostic) views that
+allow to display various amoco objects according to the underlying
+graphics Engine that is loaded (see :mod:`amoco.ui.graphics`).
+
+All views inherit from the same :class:`View` class and then
+specializes their :meth:`_vltable` method to transform an amoco object
+into a table of rows of Tokens which in turn can be used by the loaded
+engine to instanciate the graphics.
+
+The default engine :mod:`amoco.ui.graphics.term` has no specific
+graphic objects and just returns its vltable for pretty printing
+using the pygments package.
 """
 
 import re
@@ -24,37 +35,36 @@ from amoco.cas.expressions import regtype
 from amoco.ui.graphics import Engine
 from amoco.ui.render import Token, vltable, tokenrow, icons
 
+
 # -------------------------------------------------------------------------------
 
 
 class View(Engine):
-    """Class that implements common API for views.
-    A view represents an amoco element (either a block, a map or a function) as
-    a positionable "boxed" object, ie an object with (width,height) and (x,y)
+    """
+    Base class that implements common API for all views.
+
+    A view represents an amoco element (either a block, a map, a function, ...)
+    as a positionable "boxed" object, ie an object with (width,height) and (x,y)
     coords. A view is bound to the configured graphics engine through its parent
-    Engine class and offers a common API to display its object.
+    Engine class and offers a common API to display the amoco element.
 
     Args:
         of: the amoco element associated with this view.
 
     Attributes:
         of: the amoco element associated with this view.
-        obj: the engine's graphical object that represents "of".
+        obj: the engine's graphic object that represents the amoco element.
+             This is a lazy constructed object that calls the engine's builder
+             upon first attribute access. Hence, the object is never constructed
+             until it is used for the first time.
         w: the width of the view.
         h: the height of the view.
         xy (tuple[int]): the (x,y) coords of the view.
 
     Methods:
-        __str__ (str): a pretty-printed string of the amoco element.
+        __str__ (str): a pretty-printed string that represents the element.
     """
 
-    _is_block = False
-    _is_map = False
-    _is_func = False
-    _is_xfunc = False
-    _is_task = False
-    _is_emul = False
-
     def __init__(self, of=None):
         self.of = of
 
@@ -93,24 +103,77 @@ class View(Engine):
     def __str__(self):
         return self.engine.pp(self)
 
+
+# -------------------------------------------------------------------------------
+
+
+class dataView(View):
+    """
+    This class implements the view for :class:`system.core.DataIO` objects.
+
+    A dataView specializes :class:`View` by implementing the _vltable method
+    which allows to pretty print (with pygments' highlight) the hexdump of the
+    associated file/stream bytes.
+    """
+
+    def __init__(self, dataio):
+        super().__init__(of=dataio)
+        self.cur = 0
+        self.nbb = 16
+        self.nbl = 16
+
+    def hexdump(self,cur=None,nbb=None,nbl=None):
+        if cur is None: cur=self.cur
+        if nbb is None: nbb=self.nbb
+        if nbl is None: nbl=self.nbl
+        self.cur = cur
+        self.nbb = nbb
+        self.nbl = nbl
+        t = vltable()
+        t.rowparams["sep"] = icons.sep
+        for l in range(self.nbl):
+            r = [(Token.Address, "%08x"%self.cur), (Token.Column, "")]
+            data = self.of[self.cur: self.cur+self.nbb]
+            s = " ".join(["%02x"%d for d in data])
+            # add extra space
+            s = s[0:23]+" "+s[23:]
+            r.append((Token.Literal, s))
+            r.append((Token.Column,""))
+            s = "".join([chr(x) if 32<=x<=127 else '.' for x in data])
+            r.append((Token.Literal, s))
+            self.cur += self.nbb
+            t.addrow(r)
+        return t
+
+    def _vltable(self, **kargs):
+        t = vltable(**kargs)
+        t.rowparams["sep"] = icons.sep
+        # ....
+        return t
+
+
 # -------------------------------------------------------------------------------
 
 
 class blockView(View):
-    """Class that implements view of code.block objects.
-    A blockView additionnally implements the _vltable method which allows to
-    pretty print the block through ui.render.highlight method.
-    The str() representation of a blockView instance uses this pretty printer
-    through engines' pp method.
     """
+    This class implements the view for :class:`code.block` objects.
 
-    _is_block = True
+    A blockView specializes :class:`View` by implementing the _vltable method
+    which allows to pretty print the block instructions with pygments' highlight
+    or to rely on the underlying engine to build a graphic object from this table.
+    """
 
     def __init__(self, block):
         super().__init__(of=block)
 
     @staticmethod
     def instr(i, flavor=None):
+        """
+        Helper function that returns a list of tokens for the provided instr
+        with additional columns like the address of the instruction and optionally
+        also its bytecode.
+        """
         ins2 = i.toks()
         if isinstance(ins2, str):
             ins2 = [(Token.Literal, ins2)]
@@ -133,6 +196,10 @@ class blockView(View):
         return T
 
     def _vltable(self, **kargs):
+        """
+        Returns the vltable from instructions' of the block with additional
+        header/footer, address and possibly bytecode.
+        """
         T = vltable(**kargs)
         for i in self.of.instr:
             T.addrow(self.instr(i))
@@ -158,8 +225,6 @@ class mapperView(View):
     through engines' pp method.
     """
 
-    _is_map = True
-
     def __init__(self, m):
         super().__init__(of=m)
 
@@ -195,8 +260,6 @@ class mmapView(View):
     through engines' pp method.
     """
 
-    _is_map = True
-
     def __init__(self, m):
         super().__init__(of=m)
 
@@ -220,6 +283,7 @@ class mmapView(View):
             t.addrow((Token.Memory,icons.hor*8))
         return t
 
+
 # -------------------------------------------------------------------------------
 
 
@@ -229,8 +293,6 @@ class funcView(View):
     pretty print the function through engines' pp method.
     """
 
-    _is_func = True
-
     def __init__(self, func):
         from grandalf.layouts import SugiyamaLayout
 
@@ -249,7 +311,6 @@ class funcView(View):
 
 
 class xfuncView(View):
-    _is_xfunc = True
 
     def __init__(self, xfunc):
         super().__init__(of=xfunc)
@@ -257,8 +318,8 @@ class xfuncView(View):
 
 # -------------------------------------------------------------------------------
 
+
 class execView(View):
-    _is_task = True
 
     def __init__(self,of):
         super().__init__(of)
@@ -276,11 +337,13 @@ class execView(View):
                   (Token.Address, self.of.bin.__class__.__name__),
                   (Token.Column,''),
                   (Token.Address, self.of.__module__)])
+        if not more and hasattr(self.of, "title_info"):
+            more = [(Token.Alert,x) for x in self.of.title_info()]
         if more:
             r = t.rows[0]
             for x in more:
-                r.toks.append((Token.Column,''))
-                r.toks.append(x)
+                r.cols[-1].append((Token.Column,''))
+                r.cols.append([x])
             t.update()
         return t
 
@@ -315,6 +378,8 @@ class execView(View):
                     if _r._is_slc:
                         sta,sto = _r.pos,_r.pos+_r.size
                         _r = _r.x
+                    else:
+                        sta,sto = 0,_r.size
                     val = [(Token.Literal,'[ ')]
                     for pos,sz in _r._subrefs:
                         if (sta<=pos<sto) and (sz<(sto-sta)):
@@ -371,7 +436,7 @@ class execView(View):
                     tn = tn[1]
                     if tn == Token.Memory:
                         tn = Token.Address
-                        tv = re.findall('\[(0x[0-9a-zA-Z]+)\]',tv)
+                        tv = re.findall(r'\[(0x[0-9a-zA-Z]+)\]',tv)
                         if len(tv)==1:
                             tv = tv[0]
                         else:
@@ -415,7 +480,6 @@ class execView(View):
 
 
 class emulView(View):
-    _is_emul = True
 
     def __init__(self,of,frames=None):
         super().__init__(of)
@@ -503,6 +567,7 @@ class emulView(View):
                 v = self.of.task.state(x)
                 if v!=0:
                     sp.append(v)
+        sz = self.of.pc.length
         # if we have more than 1 stack registers (ebp, esp)
         if len(sp)==2:
             delta = sp[1]-sp[0]
@@ -513,10 +578,13 @@ class emulView(View):
                     delta = -delta
                 elif delta==0:
                     delta = 8
-            sz = self.of.pc.length
+                elif delta>conf.Emu.stacksize:
+                    delta = conf.Emu.stacksize
+            else:
+                delta = conf.Emu.stacksize
             t.append(str(self.of.task.view.memory(sp[0],delta//sz)))
         elif len(sp)==1:
-            t.append(str(self.of.task.view.memory(sp[0],4)))
+            t.append(str(self.of.task.view.memory(sp[0],sz)))
         else:
             logger.warning("stack pointer not found")
         return t
@@ -531,10 +599,11 @@ class emulView(View):
         t.append(self.line())
         return '\n'.join(t)
 
+
 # -------------------------------------------------------------------------------
 
+
 class archView(View):
-    _is_emul = True
 
     def __init__(self,of):
         super().__init__(of)
@@ -564,4 +633,5 @@ class archView(View):
             t.extend(self.show_subtree(root))
         return "\n".join(t)
 
+
 # -------------------------------------------------------------------------------
